@@ -16,16 +16,17 @@
  */
 package org.apache.catalina.core;
 
-
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Set;
 
 import javax.servlet.RequestDispatcher;
 import javax.servlet.ServletRequest;
 import javax.servlet.ServletRequestWrapper;
 
-
 /**
  * Wrapper around a <code>javax.servlet.ServletRequest</code> that transforms an application request object (which might
  * be the original one passed to a servlet, or might be based on the 2.3
@@ -40,13 +41,12 @@ import javax.servlet.ServletRequestWrapper;
  */
 class ApplicationRequest extends ServletRequestWrapper {
 
-
-    // ------------------------------------------------------- Static Variables
-
-
     /**
      * The set of attribute names that are special for request dispatchers.
+     *
+     * @deprecated Will be removed without replacement in Tomcat 11 onwards.
      */
+    @Deprecated
     protected static final String specials[] =
             { RequestDispatcher.INCLUDE_REQUEST_URI, RequestDispatcher.INCLUDE_CONTEXT_PATH,
                     RequestDispatcher.INCLUDE_SERVLET_PATH, RequestDispatcher.INCLUDE_PATH_INFO,
@@ -54,25 +54,13 @@ class ApplicationRequest extends ServletRequestWrapper {
                     RequestDispatcher.FORWARD_REQUEST_URI, RequestDispatcher.FORWARD_CONTEXT_PATH,
                     RequestDispatcher.FORWARD_SERVLET_PATH, RequestDispatcher.FORWARD_PATH_INFO,
                     RequestDispatcher.FORWARD_QUERY_STRING, RequestDispatcher.FORWARD_MAPPING };
-
-
-    // ----------------------------------------------------------- Constructors
-
-
-    /**
-     * Construct a new wrapped request around the specified servlet request.
-     *
-     * @param request The servlet request being wrapped
+    /*
+     * This duplicates specials but has been added to improve the performance of isSpecial().
      */
-    ApplicationRequest(ServletRequest request) {
+    private static final Set<String> specialsSet = new HashSet<>(Arrays.asList(specials));
 
-        super(request);
-        setRequest(request);
-
-    }
-
-
-    // ----------------------------------------------------- Instance Variables
+    private static final int shortestSpecialNameLength =
+            specialsSet.stream().mapToInt(String::length).min().getAsInt();
 
 
     /**
@@ -80,10 +68,19 @@ class ApplicationRequest extends ServletRequestWrapper {
      */
     protected final HashMap<String,Object> attributes = new HashMap<>();
 
+    /**
+     * Construct a new wrapped request around the specified servlet request.
+     *
+     * @param request The servlet request being wrapped
+     */
+    ApplicationRequest(ServletRequest request) {
+        super(request);
+        setRequest(request);
+    }
+
 
     // ------------------------------------------------- ServletRequest Methods
 
-
     /**
      * Override the <code>getAttribute()</code> method of the wrapped request.
      *
@@ -91,11 +88,9 @@ class ApplicationRequest extends ServletRequestWrapper {
      */
     @Override
     public Object getAttribute(String name) {
-
         synchronized (attributes) {
             return attributes.get(name);
         }
-
     }
 
 
@@ -104,11 +99,9 @@ class ApplicationRequest extends ServletRequestWrapper {
      */
     @Override
     public Enumeration<String> getAttributeNames() {
-
         synchronized (attributes) {
             return Collections.enumeration(attributes.keySet());
         }
-
     }
 
 
@@ -119,14 +112,12 @@ class ApplicationRequest extends ServletRequestWrapper {
      */
     @Override
     public void removeAttribute(String name) {
-
         synchronized (attributes) {
             attributes.remove(name);
             if (!isSpecial(name)) {
                 getRequest().removeAttribute(name);
             }
         }
-
     }
 
 
@@ -138,20 +129,34 @@ class ApplicationRequest extends ServletRequestWrapper {
      */
     @Override
     public void setAttribute(String name, Object value) {
-
         synchronized (attributes) {
             attributes.put(name, value);
             if (!isSpecial(name)) {
                 getRequest().setAttribute(name, value);
             }
         }
+    }
 
+
+    /**
+     * Is this attribute name one of the special ones that is added only for included servlets?
+     *
+     * @param name Attribute name to be tested
+     *
+     * @deprecated Will be made private in Tomcat 11 onwards.
+     */
+    @Deprecated
+    protected boolean isSpecial(String name) {
+        // Performance - see BZ 68089
+        if (name.length() < shortestSpecialNameLength) {
+            return false;
+        }
+        return specialsSet.contains(name);
     }
 
 
     // ------------------------------------------ ServletRequestWrapper Methods
 
-
     /**
      * Set the request that we are wrapping.
      *
@@ -159,7 +164,6 @@ class ApplicationRequest extends ServletRequestWrapper {
      */
     @Override
     public void setRequest(ServletRequest request) {
-
         super.setRequest(request);
 
         // Initialize the attributes for this request
@@ -172,28 +176,5 @@ class ApplicationRequest extends ServletRequestWrapper {
                 attributes.put(name, value);
             }
         }
-
     }
-
-
-    // ------------------------------------------------------ Protected Methods
-
-
-    /**
-     * Is this attribute name one of the special ones that is added only for included servlets?
-     *
-     * @param name Attribute name to be tested
-     */
-    protected boolean isSpecial(String name) {
-
-        for (String special : specials) {
-            if (special.equals(name)) {
-                return true;
-            }
-        }
-        return false;
-
-    }
-
-
 }
