@@ -51,6 +51,8 @@ import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
 import org.eclipse.jdt.internal.compiler.env.INameEnvironment;
 import org.eclipse.jdt.internal.compiler.env.NameEnvironmentAnswer;
 import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
+import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
+import org.eclipse.jdt.internal.compiler.lookup.ModuleBinding;
 import org.eclipse.jdt.internal.compiler.problem.DefaultProblemFactory;
 
 /**
@@ -141,6 +143,16 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
             public boolean ignoreOptionalProblems() {
                 return false;
             }
+
+            @Override
+            public ModuleBinding module(LookupEnvironment environment) {
+                return environment.getModule(ModuleBinding.UNNAMED);
+            }
+
+            @Override
+            public char[] getModuleName() {
+                return ModuleBinding.UNNAMED;
+            }
         }
 
         final INameEnvironment env = new INameEnvironment() {
@@ -339,6 +351,11 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                 // Java 8.
                 // This is checked against the actual version below.
                 settings.put(CompilerOptions.OPTION_Source, "24");
+            } else if (opt.equals("25")) {
+                // Constant not available in latest ECJ version shipped with
+                // Tomcat. May be supported in a snapshot build.
+                // This is checked against the actual version below.
+                settings.put(CompilerOptions.OPTION_Source, "25");
             } else {
                 log.warn(Localizer.getMessage("jsp.warning.unknown.sourceVM", opt));
                 settings.put(CompilerOptions.OPTION_Source, CompilerOptions.VERSION_1_8);
@@ -445,6 +462,12 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                 // This is checked against the actual version below.
                 settings.put(CompilerOptions.OPTION_TargetPlatform, "24");
                 settings.put(CompilerOptions.OPTION_Compliance, "24");
+            } else if (opt.equals("25")) {
+                // Constant not available in latest ECJ version shipped with
+                // Tomcat. May be supported in a snapshot build.
+                // This is checked against the actual version below.
+                settings.put(CompilerOptions.OPTION_TargetPlatform, "25");
+                settings.put(CompilerOptions.OPTION_Compliance, "25");
             } else {
                 log.warn(Localizer.getMessage("jsp.warning.unknown.targetVM", opt));
                 settings.put(CompilerOptions.OPTION_TargetPlatform, CompilerOptions.VERSION_1_8);
