@@ -20,6 +20,7 @@ import java.nio.charset.StandardCharsets;
 
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.http.CookiesWithoutEquals;
 import org.apache.tomcat.util.http.ServerCookie;
 import org.apache.tomcat.util.http.ServerCookies;
 import org.apache.tomcat.util.log.UserDataHelper;
@@ -27,35 +28,37 @@ import org.apache.tomcat.util.res.StringManager;
 
 
 /**
- * <p>Cookie header parser based on RFC6265 and RFC2109.</p>
- * <p>The parsing of cookies using RFC6265 is more relaxed that the
- * specification in the following ways:</p>
+ * <p>
+ * Cookie header parser based on RFC6265 and RFC2109.
+ * </p>
+ * <p>
+ * The parsing of cookies using RFC6265 is more relaxed that the specification in the following ways:
+ * </p>
  * <ul>
- *   <li>Values 0x80 to 0xFF are permitted in cookie-octet to support the use of
- *       UTF-8 in cookie values as used by HTML 5.</li>
- *   <li>For cookies without a value, the '=' is not required after the name as
- *       some browsers do not sent it.</li>
+ * <li>Values 0x80 to 0xFF are permitted in cookie-octet to support the use of UTF-8 in cookie values as used by HTML
+ * 5.</li>
+ * <li>For cookies without a value, the '=' is not required after the name as some browsers do not sent it.</li>
  * </ul>
- * <p>The parsing of cookies using RFC2109 is more relaxed that the
- * specification in the following ways:</p>
+ * <p>
+ * The parsing of cookies using RFC2109 is more relaxed that the specification in the following ways:
+ * </p>
  * <ul>
- *   <li>Values for the path attribute that contain a / character do not have to
- *       be quoted even though / is not permitted in a token.</li>
+ * <li>Values for the path attribute that contain a / character do not have to be quoted even though / is not permitted
+ * in a token.</li>
  * </ul>
- *
- * <p>Implementation note:<br>
- * This class has been carefully tuned to ensure that it has equal or better
- * performance than the original Netscape/RFC2109 cookie parser. Before
- * committing and changes, ensure that the TesterCookiePerformance unit test
- * continues to give results within 1% for the old and new parsers.</p>
+ * <p>
+ * Implementation note:<br>
+ * This class has been carefully tuned to ensure that it has equal or better performance than the original
+ * Netscape/RFC2109 cookie parser. Before committing and changes, ensure that the TesterCookiePerformance unit test
+ * continues to give results within 1% for the old and new parsers.
+ * </p>
  */
 public class Cookie {
 
     private static final Log log = LogFactory.getLog(Cookie.class);
     private static final UserDataHelper invalidCookieVersionLog = new UserDataHelper(log);
     private static final UserDataHelper invalidCookieLog = new UserDataHelper(log);
-    private static final StringManager sm =
-            StringManager.getManager("org.apache.tomcat.util.http.parser");
+    private static final StringManager sm = StringManager.getManager("org.apache.tomcat.util.http.parser");
 
     private static final boolean isCookieOctet[] = new boolean[256];
     private static final boolean isText[] = new boolean[256];
@@ -76,21 +79,13 @@ public class Cookie {
 
     static {
         // %x21 / %x23-2B / %x2D-3A / %x3C-5B / %x5D-7E (RFC6265)
-        // %x80 to %xFF                                 (UTF-8)
+        // %x80 to %xFF (UTF-8)
         for (int i = 0; i < 256; i++) {
-            if (i < 0x21 || i == QUOTE_BYTE || i == COMMA_BYTE ||
-                    i == SEMICOLON_BYTE || i == SLASH_BYTE || i == DEL_BYTE) {
-                isCookieOctet[i] = false;
-            } else {
-                isCookieOctet[i] = true;
-            }
+            isCookieOctet[i] = !(i < 0x21 || i == QUOTE_BYTE || i == COMMA_BYTE ||
+                i == SEMICOLON_BYTE || i == SLASH_BYTE || i == DEL_BYTE);
         }
         for (int i = 0; i < 256; i++) {
-            if (i < TAB_BYTE || (i > TAB_BYTE && i < SPACE_BYTE) || i == DEL_BYTE) {
-                isText[i] = false;
-            } else {
-                isText[i] = true;
-            }
+            isText[i] = !(i < TAB_BYTE || (i > TAB_BYTE && i < SPACE_BYTE) || i == DEL_BYTE);
         }
     }
 
@@ -100,8 +95,33 @@ public class Cookie {
     }
 
 
-    public static void parseCookie(byte[] bytes, int offset, int len,
-            ServerCookies serverCookies) {
+    /**
+     * Parse byte array as cookie header.
+     *
+     * @param bytes         Source
+     * @param offset        Start point in array
+     * @param len           Number of bytes to read
+     * @param serverCookies Structure to store results
+     *
+     * @deprecated Unused. This method will be removed in Tomcat 11 onwards.
+     */
+    @Deprecated
+    public static void parseCookie(byte[] bytes, int offset, int len, ServerCookies serverCookies) {
+        parseCookie(bytes, offset, len, serverCookies, CookiesWithoutEquals.NAME);
+    }
+
+
+    /**
+     * Parse byte array as cookie header.
+     *
+     * @param bytes                Source
+     * @param offset               Start point in array
+     * @param len                  Number of bytes to read
+     * @param serverCookies        Structure to store results
+     * @param cookiesWithoutEquals How to handle a cookie name-value-pair that does not contain an equals character
+     */
+    public static void parseCookie(byte[] bytes, int offset, int len, ServerCookies serverCookies,
+            CookiesWithoutEquals cookiesWithoutEquals) {
 
         // ByteBuffer is used throughout this parser as it allows the byte[]
         // and position information to be easily passed between parsing methods
@@ -118,8 +138,8 @@ public class Cookie {
 
         SkipResult skipResult = skipBytes(bb, VERSION_BYTES);
         if (skipResult != SkipResult.FOUND) {
-            // No need to reset position since skipConstant() will have done it
-            parseCookieRfc6265(bb, serverCookies);
+            // No need to reset position since skipBytes() will have done it
+            parseCookieRfc6265(bb, serverCookies, cookiesWithoutEquals);
             return;
         }
 
@@ -127,10 +147,10 @@ public class Cookie {
 
         skipResult = skipByte(bb, EQUALS_BYTE);
         if (skipResult != SkipResult.FOUND) {
-            // Need to reset position as skipConstant() will only have reset to
+            // Need to reset position as skipByte() will only have reset to
             // position before it was called
             bb.position(mark);
-            parseCookieRfc6265(bb, serverCookies);
+            parseCookieRfc6265(bb, serverCookies, cookiesWithoutEquals);
             return;
         }
 
@@ -147,7 +167,6 @@ public class Cookie {
                 if (b == SEMICOLON_BYTE || b == COMMA_BYTE) {
                     parseCookieRfc2109(bb, serverCookies, version);
                 }
-                return;
             } else {
                 // Unrecognised version.
                 // Ignore this header.
@@ -193,7 +212,8 @@ public class Cookie {
     }
 
 
-    private static void parseCookieRfc6265(ByteBuffer bb, ServerCookies serverCookies) {
+    private static void parseCookieRfc6265(ByteBuffer bb, ServerCookies serverCookies,
+            CookiesWithoutEquals cookiesWithoutEquals) {
 
         boolean moreToProcess = true;
 
@@ -211,7 +231,7 @@ public class Cookie {
                 skipLWS(bb);
                 value = readCookieValueRfc6265(bb);
                 if (value == null) {
-                    // Invalid cookie value. Skip to the next semi-colon
+                    // Invalid cookie value. Skip to the next semicolon
                     skipUntilSemiColon(bb);
                     logInvalidHeader(start, bb);
                     continue;
@@ -223,7 +243,7 @@ public class Cookie {
             if (skipResult == SkipResult.FOUND) {
                 // NO-OP
             } else if (skipResult == SkipResult.NOT_FOUND) {
-                // Invalid cookie. Ignore it and skip to the next semi-colon
+                // Invalid cookie. Ignore it and skip to the next semicolon
                 skipUntilSemiColon(bb);
                 logInvalidHeader(start, bb);
                 continue;
@@ -233,11 +253,22 @@ public class Cookie {
             }
 
             if (name.hasRemaining()) {
-                ServerCookie sc = serverCookies.addCookie();
-                sc.getName().setBytes(name.array(), name.position(), name.remaining());
                 if (value == null) {
-                    sc.getValue().setBytes(EMPTY_BYTES, 0, EMPTY_BYTES.length);
+                    switch (cookiesWithoutEquals) {
+                        case IGNORE: {
+                            // This name-value-pair is a NO-OP
+                            break;
+                        }
+                        case NAME: {
+                            ServerCookie sc = serverCookies.addCookie();
+                            sc.getName().setBytes(name.array(), name.position(), name.remaining());
+                            sc.getValue().setBytes(EMPTY_BYTES, 0, EMPTY_BYTES.length);
+                            break;
+                        }
+                    }
                 } else {
+                    ServerCookie sc = serverCookies.addCookie();
+                    sc.getName().setBytes(name.array(), name.position(), name.remaining());
                     sc.getValue().setBytes(value.array(), value.position(), value.remaining());
                 }
             }
@@ -245,8 +276,7 @@ public class Cookie {
     }
 
 
-    private static void parseCookieRfc2109(ByteBuffer bb, ServerCookies serverCookies,
-            int version) {
+    private static void parseCookieRfc2109(ByteBuffer bb, ServerCookies serverCookies, int version) {
 
         boolean moreToProcess = true;
 
@@ -362,10 +392,10 @@ public class Cookie {
                 sc.getName().setBytes(name.array(), name.position(), name.remaining());
                 sc.getValue().setBytes(value.array(), value.position(), value.remaining());
                 if (domain != null) {
-                    sc.getDomain().setBytes(domain.array(),  domain.position(),  domain.remaining());
+                    sc.getDomain().setBytes(domain.array(), domain.position(), domain.remaining());
                 }
                 if (path != null) {
-                    sc.getPath().setBytes(path.array(),  path.position(),  path.remaining());
+                    sc.getPath().setBytes(path.array(), path.position(), path.remaining());
                 }
             }
         }
@@ -380,7 +410,7 @@ public class Cookie {
 
 
     private static void skipLWS(ByteBuffer bb) {
-        while(bb.hasRemaining()) {
+        while (bb.hasRemaining()) {
             byte b = bb.get();
             if (b != TAB_BYTE && b != SPACE_BYTE) {
                 bb.rewind();
@@ -391,7 +421,7 @@ public class Cookie {
 
 
     private static void skipUntilSemiColon(ByteBuffer bb) {
-        while(bb.hasRemaining()) {
+        while (bb.hasRemaining()) {
             if (bb.get() == SEMICOLON_BYTE) {
                 break;
             }
@@ -400,7 +430,7 @@ public class Cookie {
 
 
     private static void skipUntilSemiColonOrComma(ByteBuffer bb) {
-        while(bb.hasRemaining()) {
+        while (bb.hasRemaining()) {
             byte b = bb.get();
             if (b == SEMICOLON_BYTE || b == COMMA_BYTE) {
                 break;
@@ -441,8 +471,7 @@ public class Cookie {
 
 
     /**
-     * Similar to readCookieValueRfc6265() but also allows a comma to terminate
-     * the value (as permitted by RFC2109).
+     * Similar to readCookieValueRfc6265() but also allows a comma to terminate the value (as permitted by RFC2109).
      */
     private static ByteBuffer readCookieValue(ByteBuffer bb) {
         boolean quoted = false;
@@ -477,8 +506,7 @@ public class Cookie {
 
 
     /**
-     * Similar to readCookieValue() but treats a comma as part of an invalid
-     * value.
+     * Similar to readCookieValue() but treats a comma as part of an invalid value.
      */
     private static ByteBuffer readCookieValueRfc6265(ByteBuffer bb) {
         boolean quoted = false;
@@ -612,8 +640,8 @@ public class Cookie {
             if (value == null) {
                 version = sm.getString("cookie.valueNotPresent");
             } else {
-                version = new String(value.bytes, value.position(),
-                        value.limit() - value.position(), StandardCharsets.UTF_8);
+                version = new String(value.bytes, value.position(), value.limit() - value.position(),
+                        StandardCharsets.UTF_8);
             }
             String message = sm.getString("cookie.invalidCookieVersion", version);
             switch (logMode) {
@@ -631,14 +659,13 @@ public class Cookie {
 
 
     /**
-     * Custom implementation that skips many of the safety checks in
-     * {@link java.nio.ByteBuffer}.
+     * Custom implementation that skips many of the safety checks in {@link java.nio.ByteBuffer}.
      */
     private static class ByteBuffer {
 
         private final byte[] bytes;
-        private int limit;
-        private int position = 0;
+        private final int limit;
+        private int position;
 
         ByteBuffer(byte[] bytes, int offset, int len) {
             this.bytes = bytes;
