@@ -66,6 +66,7 @@ public class SecureNioChannel extends NioChannel {
     protected boolean sniComplete = false;
 
     protected boolean handshakeComplete = false;
+    protected boolean needHandshakeWrap = false;
     protected HandshakeStatus handshakeStatus; //gets set by handshake
 
     protected boolean closed = false;
@@ -113,7 +114,7 @@ public class SecureNioChannel extends NioChannel {
 //===========================================================================================
 
     /**
-     * Flushes the buffer to the network, non blocking
+     * Flushes the buffer to the network, non-blocking
      * @param buf ByteBuffer
      * @return boolean true if the buffer has been emptied out, false otherwise
      * @throws IOException An IO error occurred writing data
@@ -128,7 +129,7 @@ public class SecureNioChannel extends NioChannel {
     }
 
     /**
-     * Performs SSL handshake, non blocking, but performs NEED_TASK on the same
+     * Performs SSL handshake, non-blocking, but performs NEED_TASK on the same
      * thread. Hence, you should never call this method using your Acceptor
      * thread, as you would slow down your system significantly. If the return
      * value from this method is positive, the selection key should be
@@ -137,7 +138,7 @@ public class SecureNioChannel extends NioChannel {
      * @param read boolean - true if the underlying channel is readable
      * @param write boolean - true if the underlying channel is writable
      *
-     * @return 0 if hand shake is complete, -1 if an error (other than an
+     * @return 0 if handshake is complete, -1 if an error (other than an
      *         IOException) occurred, otherwise it returns a SelectionKey
      *         interestOps value
      *
@@ -163,7 +164,7 @@ public class SecureNioChannel extends NioChannel {
             return SelectionKey.OP_WRITE; //we still have data to write
         }
 
-        SSLEngineResult handshake = null;
+        SSLEngineResult handshake;
 
         while (!handshakeComplete) {
             switch (handshakeStatus) {
@@ -218,7 +219,7 @@ public class SecureNioChannel extends NioChannel {
                             handshakeStatus = tasks();
                         }
                     } else if ( handshake.getStatus() == Status.BUFFER_UNDERFLOW ){
-                        //read more data, reregister for OP_READ
+                        //read more data, register again for OP_READ
                         return SelectionKey.OP_READ;
                     } else {
                         throw new IOException(sm.getString("channel.nio.ssl.unexpectedStatusDuringWrap", handshake.getStatus()));
@@ -301,8 +302,8 @@ public class SecureNioChannel extends NioChannel {
             throw new IOException(sm.getString("channel.nio.ssl.foundHttp"));
         }
 
-        if (log.isDebugEnabled()) {
-            log.debug(sm.getString("channel.nio.ssl.sniHostName", sc, hostName));
+        if (log.isTraceEnabled()) {
+            log.trace(sm.getString("channel.nio.ssl.sniHostName", sc, hostName));
         }
 
         sslEngine = endpoint.createSSLEngine(hostName, clientRequestedCiphers,
@@ -399,8 +400,7 @@ public class SecureNioChannel extends NioChannel {
             throw x;
         } catch (Exception cx) {
             closeSilently();
-            IOException x = new IOException(cx);
-            throw x;
+            throw new IOException(cx);
         } finally {
             if (key != null) {
                 try {
@@ -424,7 +424,7 @@ public class SecureNioChannel extends NioChannel {
      * @return the status
      */
     protected SSLEngineResult.HandshakeStatus tasks() {
-        Runnable r = null;
+        Runnable r;
         while ((r = sslEngine.getDelegatedTask()) != null) {
             r.run();
         }
@@ -463,10 +463,6 @@ public class SecureNioChannel extends NioChannel {
      */
     protected SSLEngineResult handshakeUnwrap(boolean doread) throws IOException {
 
-        if (netInBuffer.position() == netInBuffer.limit()) {
-            //clear the buffer if we have emptied it out on data
-            netInBuffer.clear();
-        }
         if (doread)  {
             //if we have data to read, read it
             int read = sc.read(netInBuffer);
@@ -475,7 +471,7 @@ public class SecureNioChannel extends NioChannel {
             }
         }
         SSLEngineResult result;
-        boolean cont = false;
+        boolean cont;
         //loop while we can perform pure SSLEngine data
         do {
             //prepare the buffer with the incoming data
@@ -483,7 +479,12 @@ public class SecureNioChannel extends NioChannel {
             //call unwrap
             getBufHandler().configureReadBufferForWrite();
             result = sslEngine.unwrap(netInBuffer, getBufHandler().getReadBuffer());
-            //compact the buffer, this is an optional method, wonder what would happen if we didn't
+            /*
+             * ByteBuffer.compact() is an optional method but netInBuffer is created from either ByteBuffer.allocate()
+             * or ByteBuffer.allocateDirect() and the ByteBuffers returned by those methods do implement compact().
+             * The ByteBuffer must be in 'read from' mode when compact() is called and will be in 'write to' mode
+             * afterwards.
+             */
             netInBuffer.compact();
             //read in the status
             handshakeStatus = result.getHandshakeStatus();
@@ -516,7 +517,7 @@ public class SecureNioChannel extends NioChannel {
      *   if ( isOpen() ) close(true); //forces a close if you timed out
      * </code></pre>
      * @throws IOException if an I/O error occurs
-     * @throws IOException if there is data on the outgoing network buffer and
+     * @throws IOException if there is data on the outgoing network buffer, and
      *                     we are unable to flush it
      */
     @Override
@@ -584,8 +585,7 @@ public class SecureNioChannel extends NioChannel {
      * @return The number of bytes read, possibly zero, or <code>-1</code> if
      *         the channel has reached end-of-stream
      * @throws IOException If some other I/O error occurs
-     * @throws IllegalArgumentException if the destination buffer is different
-     *                                  than getBufHandler().getReadBuffer()
+     * @throws IllegalStateException if the handshake was not completed
      */
     @Override
     public int read(ByteBuffer dst) throws IOException {
@@ -623,6 +623,14 @@ public class SecureNioChannel extends NioChannel {
                 //perform any tasks if needed
                 if (unwrap.getHandshakeStatus() == HandshakeStatus.NEED_TASK) {
                     tasks();
+                } else if (unwrap.getHandshakeStatus() == HandshakeStatus.NEED_WRAP) {
+                    if (getOutboundRemaining() == 0) {
+                        handshakeWrap(true);
+                    } else if (needHandshakeWrap) {
+                        throw new IOException(sm.getString("channel.nio.ssl.handshakeWrapPending"));
+                    } else {
+                        needHandshakeWrap = true;
+                    }
                 }
                 //if we need more network data, then bail out for now.
                 if (unwrap.getStatus() == Status.BUFFER_UNDERFLOW) {
@@ -663,7 +671,7 @@ public class SecureNioChannel extends NioChannel {
                 // Something else went wrong
                 throw new IOException(sm.getString("channel.nio.ssl.unwrapFail", unwrap.getStatus()));
             }
-        } while (netInBuffer.position() != 0); //continue to unwrapping as long as the input buffer has stuff
+        } while (netInBuffer.position() != 0); //continue unwrapping as long as the input buffer has stuff
         return read;
     }
 
@@ -712,6 +720,14 @@ public class SecureNioChannel extends NioChannel {
                 //perform any tasks if needed
                 if (unwrap.getHandshakeStatus() == HandshakeStatus.NEED_TASK) {
                     tasks();
+                } else if (unwrap.getHandshakeStatus() == HandshakeStatus.NEED_WRAP) {
+                    if (getOutboundRemaining() == 0) {
+                        handshakeWrap(true);
+                    } else if (needHandshakeWrap) {
+                        throw new IOException(sm.getString("channel.nio.ssl.handshakeWrapPending"));
+                    } else {
+                        needHandshakeWrap = true;
+                    }
                 }
                 //if we need more network data, then bail out for now.
                 if (unwrap.getStatus() == Status.BUFFER_UNDERFLOW) {
@@ -788,8 +804,7 @@ public class SecureNioChannel extends NioChannel {
     public int write(ByteBuffer src) throws IOException {
         checkInterruptStatus();
         if (src == this.netOutBuffer) {
-            int written = sc.write(src);
-            return written;
+            return sc.write(src);
         } else {
             // Are we closing or closed?
             if (closing || closed) {
@@ -810,6 +825,8 @@ public class SecureNioChannel extends NioChannel {
             netOutBuffer.clear();
 
             SSLEngineResult result = sslEngine.wrap(src, netOutBuffer);
+            // Call to wrap() will have included any required handshake data
+            needHandshakeWrap = false;
             // The number of bytes written
             int written = result.bytesConsumed();
             netOutBuffer.flip();
@@ -900,6 +917,6 @@ public class SecureNioChannel extends NioChannel {
     private enum OverflowState {
         NONE,
         PROCESSING,
-        DONE;
+        DONE
     }
 }
