@@ -380,8 +380,8 @@ public abstract class SocketWrapperBase<E> {
             remaining = Math.min(remaining, len);
             readBuffer.get(b, off, remaining);
 
-            if (log.isDebugEnabled()) {
-                log.debug("Socket: [" + this + "], Read from buffer: [" + remaining + "]");
+            if (log.isTraceEnabled()) {
+                log.trace("Socket: [" + this + "], Read from buffer: [" + remaining + "]");
             }
         }
         return remaining;
@@ -394,8 +394,8 @@ public abstract class SocketWrapperBase<E> {
         socketBufferHandler.configureReadBufferForRead();
         int nRead = transfer(socketBufferHandler.getReadBuffer(), to);
 
-        if (log.isDebugEnabled()) {
-            log.debug("Socket: [" + this + "], Read from buffer: [" + nRead + "]");
+        if (log.isTraceEnabled()) {
+            log.trace("Socket: [" + this + "], Read from buffer: [" + nRead + "]");
         }
         return nRead;
     }
@@ -486,7 +486,7 @@ public abstract class SocketWrapperBase<E> {
         /*
          * While the implementations for blocking and non-blocking writes are
          * very similar they have been split into separate methods:
-         * - To allow sub-classes to override them individually. NIO2, for
+         * - To allow subclasses to override them individually. NIO2, for
          *   example, overrides the non-blocking write but not the blocking
          *   write.
          * - To enable a marginally more efficient implemented for blocking
@@ -533,7 +533,7 @@ public abstract class SocketWrapperBase<E> {
         /*
          * While the implementations for blocking and non-blocking writes are
          * very similar they have been split into separate methods:
-         * - To allow sub-classes to override them individually. NIO2, for
+         * - To allow subclasses to override them individually. NIO2, for
          *   example, overrides the non-blocking write but not the blocking
          *   write.
          * - To enable a marginally more efficient implemented for blocking
@@ -631,12 +631,12 @@ public abstract class SocketWrapperBase<E> {
             while (len > 0) {
                 off = off + thisTime;
                 doWrite(false);
-                if (len > 0 && socketBufferHandler.isWriteBufferWritable()) {
+                if (socketBufferHandler.isWriteBufferWritable()) {
                     socketBufferHandler.configureWriteBufferForWrite();
                     thisTime = transfer(buf, off, len, socketBufferHandler.getWriteBuffer());
                 } else {
                     // Didn't write any data in the last non-blocking write.
-                    // Therefore the write buffer will still be full. Nothing
+                    // Therefore, the write buffer will still be full. Nothing
                     // else to do here. Exit the loop.
                     break;
                 }
@@ -881,7 +881,7 @@ public abstract class SocketWrapperBase<E> {
          */
         PENDING,
         /**
-         * Operation was pending and non blocking.
+         * Operation was pending and non-blocking.
          */
         NOT_DONE,
         /**
@@ -939,35 +939,27 @@ public abstract class SocketWrapperBase<E> {
      * all remaining data. If the operation completes inline, the
      * completion handler will not be called.
      */
-    public static final CompletionCheck COMPLETE_WRITE = new CompletionCheck() {
-        @Override
-        public CompletionHandlerCall callHandler(CompletionState state, ByteBuffer[] buffers,
-                int offset, int length) {
-            for (int i = 0; i < length; i++) {
-                if (buffers[offset + i].hasRemaining()) {
-                    return CompletionHandlerCall.CONTINUE;
-                }
+    public static final CompletionCheck COMPLETE_WRITE = (state, buffers, offset, length) -> {
+        for (int i = 0; i < length; i++) {
+            if (buffers[offset + i].hasRemaining()) {
+                return CompletionHandlerCall.CONTINUE;
             }
-            return (state == CompletionState.DONE) ? CompletionHandlerCall.DONE
-                    : CompletionHandlerCall.NONE;
         }
+        return (state == CompletionState.DONE) ? CompletionHandlerCall.DONE
+                : CompletionHandlerCall.NONE;
     };
 
     /**
      * This utility CompletionCheck will cause the write to fully write
      * all remaining data. The completion handler will then be called.
      */
-    public static final CompletionCheck COMPLETE_WRITE_WITH_COMPLETION = new CompletionCheck() {
-        @Override
-        public CompletionHandlerCall callHandler(CompletionState state, ByteBuffer[] buffers,
-                int offset, int length) {
-            for (int i = 0; i < length; i++) {
-                if (buffers[offset + i].hasRemaining()) {
-                    return CompletionHandlerCall.CONTINUE;
-                }
+    public static final CompletionCheck COMPLETE_WRITE_WITH_COMPLETION = (state, buffers, offset, length) -> {
+        for (int i = 0; i < length; i++) {
+            if (buffers[offset + i].hasRemaining()) {
+                return CompletionHandlerCall.CONTINUE;
             }
-            return CompletionHandlerCall.DONE;
         }
+        return CompletionHandlerCall.DONE;
     };
 
     /**
@@ -975,14 +967,8 @@ public abstract class SocketWrapperBase<E> {
      * to be called once some data has been read. If the operation
      * completes inline, the completion handler will not be called.
      */
-    public static final CompletionCheck READ_DATA = new CompletionCheck() {
-        @Override
-        public CompletionHandlerCall callHandler(CompletionState state, ByteBuffer[] buffers,
-                int offset, int length) {
-            return (state == CompletionState.DONE) ? CompletionHandlerCall.DONE
-                    : CompletionHandlerCall.NONE;
-        }
-    };
+    public static final CompletionCheck READ_DATA = (state, buffers, offset, length) ->
+        (state == CompletionState.DONE) ? CompletionHandlerCall.DONE : CompletionHandlerCall.NONE;
 
     /**
      * This utility CompletionCheck will cause the completion handler
